; The GIMP -- an image manipulation program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
; 
; Convolution Matrix Presets2 script  for GIMP 1.2
;
; --------------------------------------------------------------------
;   - Changelog -
; version 0.1  2001/09/30 Iccii <iccii@hotmail.com>
;     - Initial relase (has not worked yet!)
; version 0.1a 2001/10/04 Iccii <iccii@hotmail.com>
;     - Now, real initial relase (good work!) Thanks, Kajiyama
; version 0.2  2001/10/04 Iccii <iccii@hotmail.com>
;     - Divided into multi scripts for each effects
; version 0.3  2001/10/06 Iccii <iccii@hotmail.com>
;     - Now, you can control the amount of effects by adjusting Amount slider
; version 0.3a 2001/10/19 Iccii <iccii@hotmail.com>
;     - Minor bug fixes and added some types in Vibration
; version 0.4  2001/10/19 Iccii <iccii@hotmail.com>
;     - Defined the get-common-matrix-list2 function
;     - Defined the get-around-number2 function
; version 0.4a 2001/10/20 Iccii <iccii@hotmail.com>
;     - Added Angle and Desaturate option in Emboss
;
; --------------------------------------------------------------------
;     Reference Book
; http://bookweb.kinokuniya.co.jp/cgi-bin/gwfind.cgi?W-NIPS=9971628929
; --------------------------------------------------------------------
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
; 
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
; 
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;


	;; Set register common information

(set! DESCRIPTION "Set of presets for convolution matrix")
(set! AUTHOR      "Iccii <iccii@hotmail.com>")
(set! COPYRIGHT   "Iccii")
(set! DATE        "Oct, 2001")
(set! IMAGE-TYPE  "RGB* GRAY*")
(set! EDGE-LIST   (list "Extand" "Wrap" "Crop"))
(set! TYPE-LIST   (list "Type 1"  "Type 2"  "Type 3"  "Type 4"
                        "Type 5"  "Type 6"  "Type 7"  "Type 8"
                        "Type 9"  "Type 10" "Type 11" "Type 12"
                        "Type 13" "Type 14" "Type 15" "Type 16"
                        "Type 17" "Type 18" "Type 19" "Type 20"
                        "Type 21" "Type 22" "Type 23" "Type 24"
                        "Type 25" "Type 26" "Type 27" "Type 28"
                        "Type 29" "Type 30" "Type 31"
			))
			
(set! MAX 31)


	;; Set argcs

(set! argc_matrix   25)
(set! argc_channles  5)


	;; Check & Return alpha_alg and bmode

(define (get-alpha_alg-bmode drawable alpha_alg? bmode)
  (if (eqv? (car (gimp-drawable-has-alpha drawable)) TRUE)
      (list (if (eqv? alpha_alg? TRUE) 1 0) bmode) ; Return (drawable has alpha)
      (list 0 (if (= bmode 2) 0 bmode))))          ; Return (drawable has no alpha)


	;; Check channels and return channles array

(define (get-channels drawable gray? red? green? blue? alpha?)
  (let* ((channels (cons-array argc_channles 'long)))
    (if (eqv? (car (gimp-drawable-is-gray drawable)) TRUE)
        (set! gray (if (eqv? gray? TRUE) 1 0))
        (set! gray 0))
    (if (eqv? (car (gimp-drawable-is-rgb drawable)) TRUE)
        (begin
          (set! red   (if (eqv? red?   TRUE) 1 0))
          (set! green (if (eqv? green? TRUE) 1 0))
          (set! blue  (if (eqv? blue?  TRUE) 1 0)))
        (begin
          (set! red   0)
          (set! green 0)
          (set! blue  0)))
    (if (eqv? (car (gimp-drawable-has-alpha drawable)) TRUE)
        (set! alpha   (if (eqv? alpha? TRUE) 1 0))
        (set! alpha   0))
      (aset channels 0 gray)
      (aset channels 1 red)
      (aset channels 2 green)
      (aset channels 3 blue)
      (aset channels 4 alpha)
    channels)) ; Return the channel array


	;; Return matrix array

(define (get-matrix matrix-list)
  (define (list-ref l n) (nth n l))
  (let* ((count 0)
         (matrix (cons-array argc_matrix 'double)))
    (while (< count argc_matrix)
      (aset matrix count (list-ref matrix-list count))
      (set! count (+ count 1)))
    matrix)) ; Return the matrix array


	;; (local-debug-output matrix channels alpha_alg bmode)	; <- insert to debug

(define (local-debug-output matrix channels alpha_alg bmode)
  (let* ((message "5x5 Matrix:\n\n")
         (count 0))
    (while (< count argc_matrix)
      (set! message (string-append message " " 
                                   (number->string (aref matrix count))))
      (set! count (+ count 1))
      (if (or (= count 5) (= count 10) (= count 15) (= count 20))
          (set! message (string-append message "\n"))))
    (gimp-message message))

  (let* ((message "Channles\nGray Red Green Blue Alpha:\n")
         (count 0))
    (while (< count argc_channles)
      (set! message (string-append message " " 
                                   (number->string (aref channels count))))
      (set! count (+ count 1)))
    (gimp-message (string-append message "\n\n"
                                 "alpha_alg: " (number->string alpha_alg) "\n"
                                 "bmode: "     (number->string bmode)))  ))


	;; Return common matrix list

(define (get-common-matrix-list2 type b c)
  (cond
    ((= type 0)
      (list  b  b  b  b  b
             b  b  b  b  b
             b  b  c  b  b
             b  b  b  b  b
             b  b  b  b  b))	; 24

    ((= type 1)
      (list  0  0  0  0  0
             0  0  b  0  0
             0  b  c  b  0
             0  0  b  0  0
             0  0  0  0  0))	; 4
    ((= type (- MAX 1))
      (list  b  b  b  b  b
             b  b  0  b  b
             b  0  c  0  b
             b  b  0  b  b
             b  b  b  b  b))	; 20
    ((= type 2)
      (list  0  0  0  0  0
             0  b  0  b  0
             0  0  c  0  0
             0  b  0  b  0
             0  0  0  0  0))	; 4
    ((= type (- MAX 2))
      (list  b  b  b  b  b
             b  0  b  0  b
             b  b  c  b  b
             b  0  b  0  b
             b  b  b  b  b))	; 20
    ((= type 3)
      (list  0  0  b  0  0
             0  0  0  0  0
             b  0  c  0  b
             0  0  0  0  0
             0  0  b  0  0))	; 4
    ((= type (- MAX 3))
      (list  b  b  0  b  b
             b  b  b  b  b
             0  b  c  b  0
             b  b  b  b  b
             b  b  0  b  b))	; 20
    ((= type 4)
      (list  b  0  0  0  b
             0  0  0  0  0
             0  0  c  0  0
             0  0  0  0  0
             b  0  0  0  b))	; 4
    ((= type (- MAX 4))
      (list  0  b  b  b  0
             b  b  b  b  b
             b  b  c  b  b
             b  b  b  b  b
             0  b  b  b  0))	; 20

    ((= type 5)
      (list  0  0  0  0  0
             0  b  b  b  0
             0  b  c  b  0
             0  b  b  b  0
             0  0  0  0  0))	; 8
    ((= type (- MAX 5))
      (list  b  b  b  b  b
             b  0  0  0  b
             b  0  c  0  b
             b  0  0  0  b
             b  b  b  b  b))	; 16
    ((= type 6)
      (list  0  0  b  0  0
             0  b  0  b  0
             b  0  c  0  b
             0  b  0  b  0
             0  0  b  0  0))	; 8
    ((= type (- MAX 6))
      (list  b  b  0  b  b
             b  0  b  0  b
             0  b  c  b  0
             b  0  b  0  b
             b  b  0  b  b))	; 16
    ((= type 7)
      (list  b  0  0  0  b
             0  0  b  0  0
             0  b  c  b  0
             0  0  b  0  0
             b  0  0  0  b))	; 8
    ((= type (- MAX 7))
      (list  0  b  b  b  0
             b  b  0  b  b
             b  0  c  0  b
             b  b  0  b  b
             0  b  b  b  0))	; 16
    ((= type 8)
      (list  0  0  b  0  0
             0  0  b  0  0
             b  b  c  b  b
             0  0  b  0  0
             0  0  b  0  0))	; 8
    ((= type (- MAX 8))
      (list  b  b  0  b  b
             b  b  0  b  b
             0  0  c  0  0
             b  b  0  b  b
             b  b  0  b  b))	; 16
    ((= type 9)
      (list  b  0  0  0  b
             0  b  0  b  0
             0  0  c  0  0
             0  b  0  b  0
             b  0  0  0  b))	; 8
    ((= type (- MAX 9))
      (list  0  b  b  b  0
             b  0  b  0  b
             b  b  c  b  b
             b  0  b  0  b
             0  b  b  b  0))	; 16
    ((= type 10)
      (list  0  b  0  b  0
             b  0  0  0  b
             0  0  c  0  0
             b  0  0  0  b
             0  b  0  b  0))	; 8
    ((= type (- MAX 10))
      (list  b  0  b  0  b
             0  b  b  b  0
             b  b  c  b  b
             0  b  b  b  0
             b  0  b  0  b))	; 16
    ((= type 11)
      (list  b  0  b  0  b
             0  0  0  0  0
             b  0  c  0  b
             0  0  0  0  0
             b  0  b  0  b))	; 8
    ((= type (- MAX 11))
      (list  0  b  b  b  0
             b  b  b  b  b
             0  b  c  b  0
             b  b  b  b  b
             0  b  0  b  0))	; 16

    ((= type 12)
      (list  b  0  b  0  b
             0  b  0  b  0
             b  0  c  0  b
             0  b  0  b  0
             b  0  b  0  b))	; 12
    ((= type (- MAX 12))
      (list  0  b  0  b  0
             b  0  b  0  b
             0  b  c  b  0
             b  0  b  0  b
             0  b  0  b  0))	; 12
    ((= type 13)
      (list  b  b  0  b  b
             b  0  0  0  b
             0  0  c  0  0
             b  0  0  0  b
             b  b  0  b  b))	; 12
    ((= type (- MAX 13))
      (list  b  b  0  b  b
             b  b  0  b  b
             0  0  c  0  0
             b  b  0  b  b
             b  b  0  b  b))	; 12
    ((= type 14)
      (list  b  b  0  b  b
             b  0  0  0  b
             0  0  c  0  0
             b  0  0  0  b
             b  b  0  b  b))	; 12
    ((= type (- MAX 14))
      (list  0  0  b  0  0
             0  b  b  b  0
             b  b  c  b  b
             0  b  b  b  0
             0  0  b  0  0))	; 12
    ((= type 15)
      (list  0  b  0  b  0
             b  b  0  b  b
             0  0  c  0  0
             b  b  0  b  b
             0  b  0  b  0))	; 12
    ((= type (- MAX 15))
      (list  b  0  b  0  b
             0  0  b  0  0
             b  b  c  b  b
             0  0  b  0  0
             b  0  b  0  b))	; 12

    ; else
    (begin
      (list  0  0  0  0  0
             0  0  0  0  0
             0  0  1  0  0
             0  0  0  0  0
             0  0  0  0  0))
))


	;;  Return number of b

(define (get-around-number2 type)
  (cond
    ((= type  0)  24)
    ((and (<=         1  type) (>=         4  type))  4) ;  1, 4
    ((and (>= (- MAX  1) type) (<= (- MAX  4) type)) 20) ; 27,30
    ((and (<=         5  type) (>=        11  type))  8) ;  5,11
    ((and (>= (- MAX  5) type) (<= (- MAX 11) type)) 16) ; 20,26
    ((and (<=        12  type) (>=        15  type)) 12) ; 12,15
    ((and (>= (- MAX 12) type) (<= (- MAX 15) type)) 12) ; 16,19

    ; else
    (begin 0)
))









	;; V[v


(define (script-fu-convolution-presets2-sharp
			img drawable
			type a
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (set! n (get-around-number2 type))
  (set! c (+ (* a 2) 1))
  (set! divisor 1)
  (set! offset 0)
  (set! b (- (/ a (/ n 2))))

  (let* ((matrix-list (get-common-matrix-list2 type b c))
         (matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-sharp"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Sharp..."
  (string-append DESCRIPTION " (Sharp)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            TYPE-LIST
  SF-ADJUSTMENT "Amount"          '(1.0 0.1 16.0 1 1 1 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; \ȃV[v


(define (script-fu-convolution-presets2-sharp-adjustable
			img drawable
			type a c
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (set! n (get-around-number2 type))
  (set! offset 0)
  (set! b (- (/ a (/ n 2))))
  (set! divisor (+ (* b n) c))

  (let* ((matrix-list (get-common-matrix-list2 type b c))
         (matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-sharp-adjustable"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Adjustable Sharp..."
  (string-append DESCRIPTION " (Adjustable Sharp)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            TYPE-LIST
  SF-ADJUSTMENT "Amount"          '(1.0 0.1 16.0 1 1 1 0)
  SF-ADJUSTMENT "Center Value"    '(10 1 1024 1 1 0 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; ڂ


(define (script-fu-convolution-presets2-blur
			img drawable
			type a
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (set! n (get-around-number2 type))
  (set! b a)
  (set! c 1)
  (set! offset 0)
  (set! divisor (+ (* b n) c))

  (let* ((matrix-list (get-common-matrix-list2 type b c))
         (matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-blur"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Blur..."
  (string-append DESCRIPTION " (Blur)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            TYPE-LIST
  SF-ADJUSTMENT "Amount"          '(1.0 0.1 16.0 1 1 1 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; \Ȃڂ


(define (script-fu-convolution-presets2-blur-adjustable
			img drawable
			type a c
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (set! n (get-around-number2 type))
  (set! b a)
  (set! offset 0)
  (set! divisor (+ (* b n) c))

  (let* ((matrix-list (get-common-matrix-list2 type b c))
         (matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-blur-adjustable"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Adjustable Blur..."
  (string-append DESCRIPTION " (Adjustable Blur)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            TYPE-LIST
  SF-ADJUSTMENT "Amount"          '(10 1 1024 1 1 0 0)
  SF-ADJUSTMENT "Center Value"    '(10 1  256 1 1 0 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; Ԃ


(define (script-fu-convolution-presets2-vibration
			img drawable
			type a
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)


  (set! c 1)
  (set! divisor (+ (* a 2) 1))
  (set! offset 0)
  (set! matrix-list (cond
    ((= type 0)
      (list   0   0   0   0   0
              0   0   0   0   0
              0   a   c   a   0
              0   0   0   0   0
              0   0   0   0   0))
    ((= type 1)
      (list   0   0   0   0   0
              0   a   0   0   0
              0   0   c   0   0
              0   0   0   a   0
              0   0   0   0   0))
    ((= type 2)
      (list   0   0   0   0   0
              0   0   a   0   0
              0   0   c   0   0
              0   0   a   0   0
              0   0   0   0   0))
    ((= type 3)
      (list   0   0   0   0   0
              0   0   0   a   0
              0   0   c   0   0
              0   a   0   0   0
              0   0   0   0   0))
    ((= type 4)
      (list   0   0   0   0   0
              0   0   0   0   0
              a   0   c   0   a
              0   0   0   0   0
              0   0   0   0   0))
    ((= type 5)
      (list   0   a   0   0   0
              0   0   0   0   0
              0   0   c   0   0
              0   0   0   0   a
              0   0   0   0   0))
    ((= type 6)
      (list   a   0   0   0   0
              0   0   0   0   0
              0   0   c   0   0
              0   0   0   0   0
              0   0   0   0   a))
    ((= type 7)
      (list   0   a   0   0   0
              0   0   0   0   0
              0   0   c   0   0
              0   0   0   0   0
              0   0   0   a   0))
    ((= type 8)
      (list   0   0   a   0   0
              0   0   0   0   0
              0   0   c   0   0
              0   0   0   0   0
              0   0   a   0   0))
    ((= type 9)
      (list   0   0   0   a   0
              0   0   0   0   0
              0   0   c   0   0
              0   0   0   0   0
              0   a   0   0   0))
    ((= type 10)
      (list   0   0   0   0   a
              0   0   0   0   0
              0   0   c   0   0
              0   0   0   0   0
              a   0   0   0   0))
    ((= type 11)
      (list   0   0   0   0   0
              0   0   0   0   a
              0   0   c   0   0
              a   0   0   0   0
              0   0   0   0   0))
   )) ; end of cond

  (let* ((matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-vibration"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Vibration..."
  (string-append DESCRIPTION " (Vibration)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Angle"           '( "0 (small)"  "45 (small)"
                                    "90 (small)" "135 (small)"
                                     "0"  "30"  "45"  "60"
                                    "90" "120" "135" "150")
  SF-ADJUSTMENT "Amount"          '(3.0 0.1 16.0 1 1 1 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; ֊so


(define (script-fu-convolution-presets2-edge-detect
			img drawable
			type a
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (set! n (get-around-number2 type))
  (set! c (- a))
  (set! divisor 1)
  (set! offset 0)
  (set! b (/ (+ a 1) n))

  (let* ((matrix-list (get-common-matrix-list2 type b c))
         (matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-edge-detect"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Edge Detect..."
  (string-append DESCRIPTION " (Edge Detect)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            TYPE-LIST
  SF-ADJUSTMENT "Amount"          '(1.0 0.1 16.0 1 1 1 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; ֊sڂ


(define (script-fu-convolution-presets2-blur-edge
			img drawable
			type a
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (set! n (get-around-number2 type))
  (set! c (- a))
  (set! divisor 4)
  (set! offset 0)
  (set! b (/ (+ a n) n))
  (set! divisor (+ (* b n) c))

  (let* ((matrix-list (get-common-matrix-list2 type b c))
         (matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-blur-edge"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Blur Edge..."
  (string-append DESCRIPTION " (Blur Edge)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            TYPE-LIST
  SF-ADJUSTMENT "Amount"          '(1.0 0.1 16.0 1 1 1 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; G{X


(define (script-fu-convolution-presets2-emboss
			img drawable
			type desaturate? angle a
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (define (sine degree)
    (* a (sin (/ (* 2 (+ angle degree) *pi*) 360))))

  (set! divisor 1)
  (set! offset (if (eqv? desaturate? TRUE) 128 0))
  (set! c      (if (eqv? desaturate? TRUE)   0 1))
  (set! matrix-list (cond
    ((= type 0)
      (list         0          0          0          0         0
                    0 (sine 135) (sine  90) (sine  45)         0
                    0 (sine 180)          c (sine   0)         0
                    0 (sine 225) (sine 270) (sine 315)         0
                    0          0          0          0         0))
    ((= type 1)
      (list (sine 135) (sine 120) (sine  90) (sine  60) (sine  45)
            (sine 150)          0          0          0 (sine  30)
            (sine 180)          0          c          0 (sine   0)
            (sine 210)          0          0          0 (sine 330)
            (sine 225) (sine 240) (sine 270) (sine 300) (sine 315)))
    ((= type 2)
      (list (sine 135) (sine 120) (sine  90) (sine  60) (sine  45)
            (sine 150) (sine 135) (sine  90) (sine  45) (sine  30)
            (sine 180) (sine 180)          c (sine   0) (sine   0)
            (sine 210) (sine 225) (sine 270) (sine 315) (sine 330)
            (sine 225) (sine 240) (sine 270) (sine 300) (sine 315)))
   )) ; end of cond

  (let* ((matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-emboss"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Emboss..."
  (string-append DESCRIPTION " (Emboss)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            '("Type 1"  "Type 2"  "Type 3")
  SF-TOGGLE     "Desaturate"      FALSE
  SF-ADJUSTMENT "Angle"           '(45 0 360 1 45 0 0)
  SF-ADJUSTMENT "Amount"          '(1.0 0.1 16.0 1 1 1 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; G{X̃oG[V


(define (script-fu-convolution-presets2-emboss-variation
			img drawable
			type ;a
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (set! matrix-list (cond
    ((= type 0)
      (set! divisor 5)
      (set! offset 0)
      (list   0   0   0   0   0
              0  -2  -2  -2   0
              0  -1   5   1   0
              0   2   2   2   0
              0   0   0   0   0))
    ((= type 1)
      (set! divisor 1)
      (set! offset 0)
      (list   0   0   0   0   0
              0   3   0   3   0
              0   0   1   0   0
             -1  -1  -2  -1   0
              0  -1   0   0   0))
    ((= type 2)
      (set! divisor 1)
      (set! offset 0)
      (list   0   0   0   0   0
              2   0  -1  -3  -1
              1   1   1  -2   0
              1   0   1  -1   0
              0   0   2   0   0))
   )) ; end of cond

  (let* ((matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-emboss-variation"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Emboss Variation..."
  (string-append DESCRIPTION " (Emboss Variation)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            '("Type 1" "Type 2" "Type 3")
;  SF-ADJUSTMENT "Amount"          '(1.0 0.1 16.0 1 1 1 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; |X^CY


(define (script-fu-convolution-presets2-posterize
			img drawable
			type a
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (set! matrix-list (cond
    ((= type 0)
      (set! c (* a 10))
      (set! divisor 1)
      (set! offset (- (* c 9 10)))
      (list   0   0   0   0   0
              0   0   0   0   0
              0   0   c   0   0
              0   0   0   0   0
              0   0   0   0   0))
    ((= type 1)
      (set! b (* a 10))
      (set! divisor 1)
      (set! offset (- (* b 9 10)))
      (list (* b 10)        0           0           0            0
                   0        0     (* b 5)           0            0
                   0  (* b 5)     (* b 7) (- (* b 5))            0
                   0        0 (- (* b 5))           0            0
                   0        0           0           0 (- (* b 10))))
    ((= type 2)
      (set! b (* a 2))
      (set! divisor 1)
      (set! offset (- (* a 255)))
      (list   b   0   0   0   0
              0   0   0   0   0
              0   0   0   0   0
              0   0   0   0   0
              0   0   0   0   b))
    ((= type 3)
      (set! b (* a 2))
      (set! -b (- b))
      (set! c 1)
      (set! divisor c)
      (set! offset 0)
      (list   0   0   0   0   0
              0   b  -b   b   0
              0  -b   c  -b   0
              0   b  -b   b   0
              0   0   0   0   0))
   )) ; end of cond

  (let* ((matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-posterize"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Posterize..."
  (string-append DESCRIPTION " (Posterize)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            '("Type 1" "Type 2" "Type 3" "Type 4")
  SF-ADJUSTMENT "Amount"          '(1.0 0.1 16.0 1 1 1 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; F̔]


(define (script-fu-convolution-presets2-color-invert
			img drawable
			type a
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (set! divisor 1)
  (set! matrix-list (cond
    ((= type 0)
      (set! offset 255)
      (list   0   0   0   0   0
              0   0   0   0   0
              0   0  -1   0   0
              0   0   0   0   0
              0   0   0   0   0))
    ((= type 1)
      (set! b (* a 2))
      (set! -b (- b))
      (set! c -1)
      (set! offset 255)
      (list   b   0  -b   0   b
              0   0   0   0   0
             -b   0   c   0  -b
              0   0   0   0   0
              b   0  -b   0   b))
    ((= type 2)
      (set! b (- (/ a 2)))
      (set! c (- (* (/ a 2) 24) 1))
      (set! offset 255)
      (list   b   b   b   b   b
              b   b   b   b   b
              b   b   c   b   b
              b   b   b   b   b
              b   b   b   b   b))
    ((= type 3)
      (set! b a)
      (set! -b (- b))
      (set! offset 128)
      (list   b   0   0   0   0
              0   0   0   0   0
             -b   0   0   0   b
              0   0   0   0   0
              0   0   0   0  -b))
   )) ; end of cond

  (let* ((matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-color-invert"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Color Invert..."
  (string-append DESCRIPTION " (Color Invert)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            '("Type 1" "Type 2" "Type 3" "Type 4")
  SF-ADJUSTMENT "Amount"          '(1.0 0.1 16.0 1 1 1 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)









	;; ̑


(define (script-fu-convolution-presets2-others
			img drawable
			type ;a
			alpha_alg? b_mode
			gray? red? green? blue? alpha?)

  (set! matrix-list (cond
    ((= type 0)
      (set! divisor 1)
      (set! offset -7000)
      (list   0   0  100   0   0
              0   0    0   0   0
            100   0 -330   0 100
              0   0    0   0   0
              0   0  100   0   0))
    ((= type 1)
      (set! divisor 10)
      (set! offset 290)
      (list   0   1   0   0   0
              1   2   1   0   0
             -2   0  -2   0  -1
              0   0   0  -2   0
              0   0  -1   0  -3))
    ((= type 2)
      (set! divisor 1)
      (set! offset 100)
      (list  -1  -1  -1  -1  -1
             -1  -1  -1  -1  -1
             -1  -1  24  -1  -1
             -1  -1  -1  -1  -1
             -1  -1  -1  -1  -1))
    ((= type 3)
      (set! divisor 1)
      (set! offset 0)
      (list   1   1   1   1   1
              1   1   1   1   1
              1   1 -23   1   1
              1   1   1   1   1
              1   1   1   1   1))
    ((= type 4)
      (set! divisor 1)
      (set! offset 0)
      (list   0   0   0   0   0
              0   0   1   3   1
              0  -1  -1   1   0
              0  -2  -1   1   0
              0  -2   0   1   0))
    ((= type 5)
      (set! divisor 1)
      (set! offset 0)
      (list  -1   0   6   0  -3
              0   0   0   0   0
              4   0  -7   0   4
              0   0   0   0   0
             -3   0   6   0  -5))
    ((= type 6)
      (set! divisor 1)
      (set! offset 128)
      (list   0   0   0   0   0
              0   1   1   0   0
              0   1   0  -1   0
              0   0  -1  -1   0
              0   0   0   0   0))
    ((= type 7)
      (set! divisor 1)
      (set! offset 0)
      (list   0   0   0   0   0
              0   1   1   0   0
              0   1   1  -1   0
              0   0  -1  -1   0
              0   0   0   0   0))
    ((= type 8)
      (set! divisor 1)
      (set! offset 0)
      (list   0   2   0   0   1
              2   0  -4   0   0
              0  -5   5   5   0
              0   0  -4   0   0
              0   0  -2   0   0))
    ((= type 9)
      (set! divisor 1)
      (set! offset 128)
      (list   0   0   0   0   0
              0  -1  -1  -1   0
              0  -1   8  -1   0
              0  -1  -1  -1   0
              0   0   0   0   0))
    ((= type 10)
      (set! divisor 1)
      (set! offset -128)
      (list   0   0   0   0   0
              0   1   1   1   0
              0   1  -6   1   0
              0   1   1   1   0
              0   0   0   0   0))
    ((= type 11)
      (set! divisor 49)
      (set! offset 0)
      (list   9   0   3   0   9
              0   0   0   0   0
              3   0   1   0   3
              0   0   0   0   0
              9   0   3   0   9))
    ((= type 12)
      (set! divisor 1)
      (set! offset 0)
      (list   0   0   0   0   0
             -1   0  -5   0  -1
              1   0   0   0   1
              1   0   5   0   1
              0   0   0   0   0))
    ((= type 13)
      (set! divisor 2)
      (set! offset 50)
      (list   0  -3   0  -3   0
              1   0   0   0   2
              0  -3   1   3   0
              2   0   0   0   0
              0  -1   0   3   0))
    ;; else
    (begin
      (set! divisor 1)
      (set! offset 0)
      (list   0   0   0   0   0
              0   0   0   0   0
              0   0   1   0   0
              0   0   0   0   0
              0   0   0   0   0))
   )) ; end of cond

  (let* ((matrix (get-matrix matrix-list))
         (channels (get-channels drawable gray? red? green? blue? alpha?))
         (listed (get-alpha_alg-bmode drawable alpha_alg? b_mode))
         (alpha_alg (car listed))
         (bmode (cadr listed)))

    (plug-in-convmatrix 1 img drawable argc_matrix matrix alpha_alg
                        divisor offset argc_channles channels bmode)
    (gimp-displays-flush)))

(script-fu-register
  "script-fu-convolution-presets2-others"
  "<Image>/Filters/Generic/Convolution Matrix Presets2/Others..."
  (string-append DESCRIPTION " (Others)")
  AUTHOR
  COPYRIGHT
  DATE
  IMAGE-TYPE
  SF-IMAGE      "Image"           0
  SF-DRAWABLE   "Drawable"        0
  SF-OPTION     "Type"            '("Type 1" "Type 2" "Type 3" "Type 4"
                                    "Type 5" "Type 6" "Emboss" "Colored Emboss"
                                    "Edge Lighting" "Sharp" "Edge Detection"
                                    "Blur with Direction" "Type 13" "Type 14")
;  SF-ADJUSTMENT "Amount"          '(1.0 0.1 16.0 1 1 1 0)
  SF-TOGGLE     "Alpha-weighting" TRUE
  SF-OPTION     "Edge"            EDGE-LIST
  SF-TOGGLE     "Gray Channel"    FALSE
  SF-TOGGLE     "Red Channel"     TRUE
  SF-TOGGLE     "Green Channel"   TRUE
  SF-TOGGLE     "Blue Channel"    TRUE
  SF-TOGGLE     "Alpha Channel"   FALSE)
