; The GIMP -- an image manipulation program
; Copyright (C) 1995 Spencer Kimball and Peter Mattis
; 
; Koch line script  for GIMP 1.2
; Copyright (C) 2001 Iccii <iccii@hotmail.com>
; 
; --------------------------------------------------------------------
; version 0.1  by Iccii <iccii@hotmail.com>
;     - Initial relase
;
; --------------------------------------------------------------------
;     Reference Book
; http://www.ascii.co.jp/books/detail/4-7561/4-7561-3809-8.html
; --------------------------------------------------------------------
; 
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


(define (script-fu-koch-line size line-color bg-color iteration brush)

  (let* ((brush-old (car (gimp-brushes-get-brush)))
	 (old-fg (car (gimp-palette-get-foreground)))
         (width size)
         (height (+ size (/ size 7.25)))
         (R (/ size 2))
         (initial-x0 (+ (/ width 2)  (* (cos (/ (* 2 *pi* 90) 360)) R)))
         (initial-y0 (- (/ height 2) (* (sin (/ (* 2 *pi* 90) 360)) R)))
         (initial-x1 (+ (/ width 2)  (* (cos (/ (* 2 *pi* 210) 360)) R)))
         (initial-y1 (- (/ height 2) (* (sin (/ (* 2 *pi* 210) 360)) R)))
         (initial-x2 (+ (/ width 2)  (* (cos (/ (* 2 *pi* 330) 360)) R)))
         (initial-y2 (- (/ height 2) (* (sin (/ (* 2 *pi* 330) 360)) R)))
         (img (car (gimp-image-new width height RGB)))
         (bg-layer (car (gimp-layer-new img width height
                                        RGB_IMAGE "Background" 100 NORMAL-MODE)))
         (cos60 (cos (/ (* *pi* 60) 180)))
         (sin60 (sin (/ (* *pi* 60) 180))))

	;; draw koch line

    (define (draw-line x0 y0 x1 y1 iter)
      (if (> iter 1)
	;; create new branch
          (let* ((xa (/ (+ (* 2 x0) x1) 3))
                 (ya (/ (+ (* 2 y0) y1) 3))
                 (xb (/ (+ (* 2 x1) x0) 3))
                 (yb (/ (+ (* 2 y1) y0) 3))
                 (xc (+ (- (* (- xb xa) cos60) (* (- yb ya) sin60)) xa))
                 (yc (+ (+ (* (- xb xa) sin60) (* (- yb ya) cos60)) ya)))
            (draw-line x0 y0 xa ya (- iter 1))
            (draw-line xa ya xc yc (- iter 1))
            (draw-line xc yc xb yb (- iter 1))
            (draw-line xb yb x1 y1 (- iter 1)))
	;; draw line
          (let* ((strokes (cons-array 4 'double)))
             (aset strokes 0 x0)
             (aset strokes 1 y0)
             (aset strokes 2 x1)
             (aset strokes 3 y1)
             (gimp-paintbrush bg-layer 0 4 strokes CONTINUOUS 0))
      ) ; end of if  (> iter 1)
    ) ; end of define draw line


	;; Main

    (gimp-image-undo-disable img)
    (gimp-palette-set-foreground bg-color)
    (gimp-brushes-set-brush (car brush))
    (gimp-drawable-fill bg-layer FG-IMAGE-FILL)
    (gimp-image-add-layer img bg-layer -1)
    (gimp-palette-set-foreground line-color)

    (draw-line initial-x0 initial-y0 initial-x1 initial-y1 iteration)
    (draw-line initial-x1 initial-y1 initial-x2 initial-y2 iteration)
    (draw-line initial-x2 initial-y2 initial-x0 initial-y0 iteration)

    (gimp-brushes-set-brush brush-old)
    (gimp-palette-set-foreground old-fg)
    (gimp-image-undo-enable img)
    (gimp-display-new img)
    (gimp-displays-flush)

  ) ; end of let*
)

(script-fu-register
  "script-fu-koch-line"
  ;"<Toolbox>/Xtns/Script-Fu/Patterns/Koch Line..."
  "<Toolbox>/Xtns/Script-Fu/Tests/Koch Line..."
  "Render the Koch line"
  "Iccii <iccii@hotmail.com>"
  "Iccii"
  "Jun, 2001"
  ""
  SF-ADJUSTMENT	_"Image Width"      '(345 1 1024 1 10 0 1)
  SF-COLOR      "Line Color"        '(  0   0   0)
  SF-COLOR      _"Background Color" '(255 255 255)
  SF-ADJUSTMENT "Iteration"         '(4 1 8 1 2 0 1)
  SF-BRUSH      _"Use Brush"        '("Circle (03)" 1.0 44 0)
)
